/***************************************************************************
***
***    Copyright 2008  Hon Hai Precision Ind. Co. Ltd.
***    All Rights Reserved.
***
****************************************************************************/

#include <linux/module.h>
#include <linux/init.h>
#include <linux/fs.h>
#include <linux/miscdevice.h>
#include <asm/uaccess.h>

#include <typedefs.h>
#include <bcmutils.h>
#include <siutils.h>
#include <bcmdevs.h>

#include "wps_led.h"

//#define _DEBUG

static int wps_led_major;
devfs_handle_t wps_leddev_handle;

extern int wps_led_pattern;
extern int wps_led_state;
extern int is_wl_secu_mode; 

static int
wps_led_open(struct inode *inode, struct file * file)
{
    MOD_INC_USE_COUNT;
    return 0;
}

static int
wps_led_release(struct inode *inode, struct file * file)
{
    MOD_DEC_USE_COUNT;
    return 0;
}

static int
wps_led_ioctl(struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg)
{
    if (arg)
        is_wl_secu_mode = 1;
    else
        is_wl_secu_mode = 0;

    switch (cmd)
    {
        case WPS_LED_BLINK_NORMAL:
            wps_led_state = 1;
#ifdef _DEBUG
            printk("%s: blink normal\n", __FUNCTION__);
#endif
            break;

        case WPS_LED_BLINK_QUICK:
            wps_led_state = 2;
#ifdef _DEBUG
            printk("%s: blink WPS\n", __FUNCTION__);
#endif
            break;

        case WPS_LED_BLINK_OFF:
            /* wps_led_state will change to 0 automatically after
             * blinking a few seconds if it's 2 currently
             */
            if (wps_led_state != 2)
                wps_led_state = 0;
#ifdef _DEBUG
            printk("%s: blink OFF\n", __FUNCTION__);
#endif
            break;

        case WPS_LED_CHANGE_GREEN:
            break;

        case WPS_LED_CHANGE_AMBER:
            break;

        case WPS_LED_BLINK_QUICK2:
            wps_led_state = 3;
            break;
        case WPS_LED_BLINK_AP_LOCKDOWN:
            wps_led_state = 4;
            break;

        default:
            break;
    }

    return 0;

}
static struct file_operations wps_led_fops = {
    owner:      THIS_MODULE,
    open:       wps_led_open,
    release:    wps_led_release,
    ioctl:      wps_led_ioctl,
    };

static int __init
wps_led_init(void)
{
    if ((wps_led_major = devfs_register_chrdev(0, "wps_led", &wps_led_fops)) < 0)
        return wps_led_major;

    wps_leddev_handle = devfs_register(NULL, "wps_led", DEVFS_FL_DEFAULT,
                                    wps_led_major, 0, S_IFCHR | S_IRUGO | S_IWUGO,
                                    &wps_led_fops, NULL);

    return 0;
}

static void __exit
wps_led_exit(void)
{
    if (wps_leddev_handle != NULL)
        devfs_unregister(wps_leddev_handle);
    wps_leddev_handle = NULL;
    devfs_unregister_chrdev(wps_led_major, "wps_led");
}

module_init(wps_led_init);
module_exit(wps_led_exit);
